package fr.smallcrew.utils;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import static com.google.common.base.Throwables.propagate;
import static com.google.common.collect.Maps.newHashMap;

public class MailerHelper {
  public static final String PREFIX = "mailer.";
  private final Mailer mailer;
  private final Map<String, Object> properties;

  public MailerHelper(Mailer mailer, CustomPropertyPlaceholderConfigurer configurer) {
    this.mailer = mailer;
    this.properties = getMailerProperties(configurer.getProperties());
  }

  public void send(String email, String template, Locale locale) {
    send(email, template, locale, new HashMap<String, Object>());
  }

  public void send(String email, String template, Locale locale, Map<String, Object> data) {
    try {
      mailer.mailTo(email)
        .putAll(properties)
        .putAll(data)
        .withLocale(locale)
        .withMsgTemplate(template)
        .putAll(data)
        .send();
    } catch (Exception e) {
      throw propagate(e);
    }
  }

  private Map<String, Object> getMailerProperties(Map<String, String> configurer) {
    Map<String, Object> properties = newHashMap();
    for (String key : configurer.keySet()) {
      if (key.startsWith(PREFIX)) {
        properties.put(key.replace(PREFIX, ""), configurer.get(key));
      }
    }
    return properties;
  }
}
