package fr.smallcrew.utils;

import freemarker.cache.URLTemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.Template;

import java.io.StringWriter;
import java.net.URL;
import java.util.Locale;
import java.util.Map;

import static com.google.common.base.Throwables.propagate;
import static com.google.common.collect.Maps.newHashMap;

public class TemplateHelper {
  private static final Configuration cfg;
  private final Map<String, Object> data;
  private final String name;
  private Locale locale;

  static {
    cfg = new Configuration();
    cfg.setTemplateLoader(new URLTemplateLoader() {
      @Override
      protected URL getURL(String name) {
        return getClass().getClassLoader().getResource(String.format("template/%s.ftl", name));
      }
    });
  }

  private TemplateHelper(String name) {
    this.data = newHashMap();
    this.name = name;
  }

  public static TemplateHelper template(String name) {
    return new TemplateHelper(name);
  }

  public TemplateHelper withLocale(Locale locale) {
    this.locale = locale;
    return this;
  }

  public TemplateHelper put(String name, Object value) {
    data.put(name, value);
    return this;
  }

  public TemplateHelper putAll(Map<String, Object> data) {
    this.data.putAll(data);
    return this;
  }

  public String asString() {
    try {
      checkLocale();
      Template template = null;
      try {
        template = cfg.getTemplate(name, locale);
      } catch (Exception e) {
        if (!locale.equals(Locale.ENGLISH)) {
          template = cfg.getTemplate(name, Locale.ENGLISH);
        }
      }
      StringWriter out = new StringWriter();
      template.process(data, out);
      return out.toString();
    } catch (Exception e) {
      throw propagate(e);
    }
  }

  private void checkLocale() {
    if (locale == null) {
      locale = Locale.ENGLISH;
    }
  }
}
