package fr.smallcrew.utils;

import org.apache.commons.io.IOUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import static com.google.common.base.Throwables.propagate;

public class XMLUtils {
  private static final XPathFactory xpathFactory = XPathFactory.newInstance();
  private static final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
  private static final TransformerFactory transformerFactory = TransformerFactory.newInstance();

  public static Document buildDocumentFromXML(String xml) {
    try {
      DocumentBuilder builder = documentBuilderFactory.newDocumentBuilder();
      InputSource is = new InputSource(new StringReader(xml));
      return builder.parse(is);
    } catch (Exception e) {
      throw propagate(e);
    }
  }

  public static Document buildDocumentFromFile(File file) {
    try {
      DocumentBuilder builder = documentBuilderFactory.newDocumentBuilder();
      InputSource is = new InputSource(new FileReader(file));
      return builder.parse(is);
    } catch (Exception e) {
      throw propagate(e);
    }
  }

  public static Document buildDocumentFromURL(URL url) {
    try {
      DocumentBuilder builder = documentBuilderFactory.newDocumentBuilder();
      return builder.parse(url.openStream());
    } catch (Exception e) {
      throw propagate(e);
    }
  }

  public static XPathFactory getXPathFactory() {
    return xpathFactory;
  }

  public static String toString(Node node) {
    Writer writer = null;
    String xmlString = null;
    try {
      Transformer transformer = transformerFactory.newTransformer();
      transformer.setOutputProperty(OutputKeys.INDENT, "yes");
      transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
      StreamResult result = new StreamResult(new StringWriter());
      DOMSource source = new DOMSource(node);
      transformer.transform(source, result);
      writer = result.getWriter();
      xmlString = writer.toString();
    } catch (Exception e) {
      throw propagate(e);
    } finally {
      IOUtils.closeQuietly(writer);
    }
    return xmlString;
  }

  @SuppressWarnings("unchecked")
  public static <T> T evaluate(String q, Node node, QName qName) {
    try {
      XPath xpath = XMLUtils.getXPathFactory().newXPath();
      return (T) xpath.evaluate(q, node, qName);
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  public static Node buildXmlNode(String xml) {
    ByteArrayInputStream byteArrayInputStream = null;
    try {
      byteArrayInputStream = new ByteArrayInputStream(xml.getBytes());
      return documentBuilderFactory.newDocumentBuilder().parse(byteArrayInputStream).getDocumentElement();
    } catch (Exception e) {
      throw propagate(e);
    } finally {
      IOUtils.closeQuietly(byteArrayInputStream);
    }
  }

  public static List<Node> getNodeList(String query, Node node) {
    NodeList nodeList = evaluate(query, node, XPathConstants.NODESET);
    List<Node> nodes = new ArrayList<Node>();
    for (int i = 0; i < nodeList.getLength(); i++) {
      nodes.add(nodeList.item(i));
    }
    return nodes;
  }

  public static String getStringValue(String query, Node node) {
    return evaluate(query, node, XPathConstants.STRING);
  }

  public static Node getNode(String query, Node node) {
    return evaluate(query, node, XPathConstants.NODE);
  }

  public static int getIntValue(String query, Document value) {
    return Integer.parseInt(getStringValue(query, value));
  }
}
